@@ -3,9 +3,10 @@
 """
 from __future__ import absolute_import, division, print_function
 
-
+import os
 import time
 import random
+import datetime
 
 
 # Import ioflo libs
@@ -17,6 +18,7 @@ console = getConsole()
 
 TIME1970 = long(2208988800) #offset secs between SNTP epoch=1900 & unix epoch=1970
 
+
 def totalSeconds(td):
     """ Compute total seconds for datetime.timedelta object
         needed for python 2.6
@@ -25,13 +27,51 @@ def totalSeconds(td):
 
 TotalSeconds = totalSeconds
 
-def tuuid(stamp=None):
+def timestamp(dt=None):
+    """
+    Returns float posix timestamp at dt if given else now
+    Only TZ aware in python 3.2+
+    """
+    if dt is None:
+        if hasattr(datetime, "timezone"):
+            dt = datetime.datetime.now(datetime.timezone.utc)  # make it aware
+        else:
+            dt = datetime.datetime.utcnow()  # not aware
+
+    if hasattr(dt, "timestamp"):  # only in python3.3+
+        return dt.timestamp()
+    elif hasattr(datetime, "timezone"): # only in python3.2+
+        return (dt - datetime.datetime(1970, 1, 1, tzinfo=datetime.timezone.utc)).total_seconds()
+    else: # not tz aware
+        return (dt - datetime.datetime(1970, 1, 1)).total_seconds()
+
+def iso8601(dt=None, aware=False):
+    """
+    Returns string datetime stamp in iso 8601 format from datetime object dt
+    If dt is missing and aware then use now(timezone.utc) else utcnow() naive
+    YYYY-MM-DDTHH:MM:SS.mmmmmm which is strftime '%Y-%m-%dT%H:%M:%S.%f'
+    Only TZ aware in python 3.2+
+    """
+    if dt is None:
+        if aware and hasattr(datetime, "timezone"):
+            dt = datetime.datetime.now(datetime.timezone.utc)  # make it aware
+        else:  # naive
+            dt = datetime.datetime.utcnow()  # naive
+
+    return(dt.isoformat())
+
+def tuuid(stamp=None, prefix=None):
     """
     Returns lexocographically sortable TUUID  (Time Universal Unique Identifier)
-    that as hex formated string of length 24
+    that is hex formated string of length 24
     stamp is float time since unix epoch (1970-1-1) as in time.time()
     If stamp not provided uses current system time UTC
 
+    prefix is optional prefix string that is prepended to tuid with '_'
+    for example if prefix is 'm' then tuiid looks like
+    'm_0000014ddf1f2f9c_5e36738'
+    The length of the tuuid is now 24 + len(prefix) +1
+
     Format of of TUUID is hex string of form stamphex_randombyteshex
     Example:
     '0000014ddf1f2f9c_5e36738'
@@ -53,12 +93,18 @@ def tuuid(stamp=None):
 
     Uses random.SystemRandom which is crytographically random
     """
-    stamp = stamp if stamp is not None else time.mktime(time.gmtime())
+    parts = []
+    if prefix is not None:
+        parts.append(prefix)
+
+    stamp = stamp if stamp is not None else timestamp()
     stamp = int(stamp * 1000000)
     stamp = "{0:016x}".format(stamp)[-16:]
+    parts.append(stamp)
     randomized = random.SystemRandom().randint(0, 0xFFFFFFF)
     randomized = "{0:07x}".format(randomized)[-7:]
-    return ("{0}_{1}".format(stamp, randomized))
+    parts.append(randomized)
+    return ("_".join(parts))
 
 
 class Timer(object):
