@@ -10,6 +10,7 @@ import socket
 import errno
 import platform
 from collections import deque
+from binascii import hexlify
 
 try:
     import ssl
@@ -21,6 +22,7 @@ from ...aid.sixing import *
 from ...aid.odicting import odict
 from ...aid.timing import StoreTimer
 from ...aid.consoling import getConsole
+from .. import aioing
 from ...base import storing
 
 console = getConsole()
@@ -194,8 +196,22 @@ class Incomer(object):
         try:
             data = self.cs.recv(self.bs)
         except socket.error as ex:
-            if ex.errno in (errno.EAGAIN, errno.EWOULDBLOCK):
+            # ex.args[0] is always ex.errno for better compat
+            if ex.args[0] in (errno.EAGAIN, errno.EWOULDBLOCK):
                 return None
+            elif ex.args[0] in (errno.ECONNRESET,
+                                errno.ENETRESET,
+                                errno.ENETUNREACH,
+                                errno.EHOSTUNREACH,
+                                errno.ENETDOWN,
+                                errno.EHOSTDOWN,
+                                errno.ETIMEDOUT,
+                                errno.ECONNREFUSED):
+                emsg = ("socket.error = {0}: Incomer at {1} while receiving "
+                        "from {2}\n".format(ex, self.ca, self.ha))
+                console.profuse(emsg)
+                self.cutoff = True  # this signals need to close/reopen connection
+                return bytes()  # data empty
             else:
                 emsg = ("socket.error = {0}: Incomer at {1} while receiving"
                         " from {2}\n".format(ex, self.ha, self.ca))
@@ -204,8 +220,12 @@ class Incomer(object):
 
         if data:  # connection open
             if console._verbosity >= console.Wordage.profuse:  # faster to check
-                cmsg = ("Incomer at {0} received from {1}\n"
-                       "{2}\n".format(self.ha, self.ca, data.decode("UTF-8")))
+                try:
+                    load = data.decode("UTF-8")
+                except UnicodeDecodeError as ex:
+                    load = "0x{0}".format(hexlify(data).decode("ASCII"))
+                cmsg = ("Incomer at {0}, received from {1}:\n------------\n"
+                        "{2}\n\n".format(self.ha, self.ca, load))
                 console.profuse(cmsg)
 
             if self.wlog:  # log over the wire rx
@@ -270,8 +290,23 @@ class Incomer(object):
         try:
             result = self.cs.send(data) #result is number of bytes sent
         except socket.error as ex:
-            result = 0
-            if ex.errno not in (errno.EAGAIN, errno.EWOULDBLOCK):
+            # ex.args[0] is always ex.errno for better compat
+            if ex.args[0] in (errno.EAGAIN, errno.EWOULDBLOCK):
+                result = 0  # blocked try again
+            elif ex.args[0] in (errno.ECONNRESET,
+                                errno.ENETRESET,
+                                errno.ENETUNREACH,
+                                errno.EHOSTUNREACH,
+                                errno.ENETDOWN,
+                                errno.EHOSTDOWN,
+                                errno.ETIMEDOUT,
+                                errno.ECONNREFUSED):
+                emsg = ("socket.error = {0}: Outgoer at {1} while sending "
+                        "to {2} \n".format(ex, self.ca, self.ha))
+                console.profuse(emsg)
+                self.cutoff = True  # this signals need to close/reopen connection
+                result = 0
+            else:
                 emsg = ("socket.error = {0}: Incomer at {1} while "
                         "sending to {2}\n".format(ex, self.ha, self.ca))
                 console.profuse(emsg)
@@ -279,9 +314,12 @@ class Incomer(object):
 
         if result:
             if console._verbosity >=  console.Wordage.profuse:
-                cmsg = ("Incomer at {0} sent to {1}, {2} bytes\n"
-                        "{3}\n".format(self.ha, self.ha , result,
-                                       data[:result].decode('UTF-8')))
+                try:
+                    load = data[:result].decode("UTF-8")
+                except UnicodeDecodeError as ex:
+                    load = "0x{0}".format(hexlify(data[:result]).decode("ASCII"))
+                cmsg = ("Incomer at {0}, sent {1} bytes to {2}:\n------------\n"
+                        "{3}\n\n".format(self.ha, result, self.ca, load))
                 console.profuse(cmsg)
 
             if self.wlog:
@@ -428,19 +466,38 @@ class IncomerTls(Incomer):
         """
         try:
             data = self.cs.recv(self.bs)
-        except ssl.SSLError as ex:
-            if ex.errno in (ssl.SSL_ERROR_WANT_READ, ssl.SSL_ERROR_WANT_WRITE):
-                return None
+        except socket.error as ex:  # ssl.SSLError is a subtype of socket.error
+            # ex.args[0] is always ex.errno for better compat
+            if  ex.args[0] in (ssl.SSL_ERROR_WANT_READ, ssl.SSL_ERROR_WANT_WRITE):
+                return None  # blocked waiting for data
+            elif ex.args[0] in (errno.ECONNRESET,
+                                errno.ENETRESET,
+                                errno.ENETUNREACH,
+                                errno.EHOSTUNREACH,
+                                errno.ENETDOWN,
+                                errno.EHOSTDOWN,
+                                errno.ETIMEDOUT,
+                                errno.ECONNREFUSED,
+                                ssl.SSLEOFError):
+                emsg = ("socket.error = {0}: IncomerTLS at {1} while receiving"
+                        " from {2}\n".format(ex, self.ha, self.ca))
+                console.profuse(emsg)
+                self.cutoff = True  # this signals need to close/reopen connection
+                return bytes()  # data empty
             else:
-                emsg = ("SSLError = {0}: IncomerTLS at {1} while receiving"
+                emsg = ("socket.error = {0}: IncomerTLS at {1} while receiving"
                         " from {2}\n".format(ex, self.ha, self.ca))
                 console.profuse(emsg)
                 raise  # re-raise
 
         if data:  # connection open
             if console._verbosity >= console.Wordage.profuse:  # faster to check
-                cmsg = ("Incomer at {0} received from {1}\n"
-                       "{2}\n".format(self.ha, self.ca, data.decode("UTF-8")))
+                try:
+                    load = data.decode("UTF-8")
+                except UnicodeDecodeError as ex:
+                    load = "0x{0}".format(hexlify(data).decode("ASCII"))
+                cmsg = ("Incomer at {0}, received from {1}:\n------------\n"
+                        "{2}\n\n".format(self.ha, self.ca, load))
                 console.profuse(cmsg)
 
             if self.wlog:  # log over the wire rx
@@ -460,19 +517,38 @@ class IncomerTls(Incomer):
         """
         try:
             result = self.cs.send(data) #result is number of bytes sent
-        except ssl.SSLError as ex:
-            result = 0
-            if ex.errno not in (ssl.SSL_ERROR_WANT_READ, ssl.SSL_ERROR_WANT_WRITE):
-                emsg = ("SSLError = {0}: IncomerTLS at {1} while "
+        except socket.error as ex:  # ssl.SSLError is a subtype of socket.error
+            # ex.args[0] is always ex.errno for better compat
+            if ex.args[0] in (ssl.SSL_ERROR_WANT_READ, ssl.SSL_ERROR_WANT_WRITE):
+                result = 0  # blocked try again
+            elif ex.args[0] in (errno.ECONNRESET,
+                                errno.ENETRESET,
+                                errno.ENETUNREACH,
+                                errno.EHOSTUNREACH,
+                                errno.ENETDOWN,
+                                errno.EHOSTDOWN,
+                                errno.ETIMEDOUT,
+                                errno.ECONNREFUSED,
+                                ssl.SSLEOFError):
+                emsg = ("socket.error = {0}: IncomerTLS at {1} while "
+                        "sending to {2}\n".format(ex, self.ha, self.ca))
+                console.profuse(emsg)
+                self.cutoff = True  # this signals need to close/reopen connection
+                result = 0
+            else:
+                emsg = ("socket.error = {0}: IncomerTLS at {1} while "
                         "sending to {2}\n".format(ex, self.ha, self.ca))
                 console.profuse(emsg)
                 raise
 
         if result:
             if console._verbosity >=  console.Wordage.profuse:
-                cmsg = ("Incomer at {0} sent to {1}, {2} bytes\n"
-                        "{3}\n".format(self.ha, self.ha , result,
-                                       data[:result].decode('UTF-8')))
+                try:
+                    load = data[:result].decode("UTF-8")
+                except UnicodeDecodeError as ex:
+                    load = "0x{0}".format(hexlify(data[:result]).decode("ASCII"))
+                cmsg = ("Incomer at {0}, sent {1} bytes to {2}:\n------------\n"
+                        "{3}\n\n".format(self.ha, result, self.ca, load))
                 console.profuse(cmsg)
 
             if self.wlog:
@@ -510,9 +586,11 @@ class Acceptor(object):
         eha = eha or self.ha
         if eha:
             host, port = eha
-            host = socket.gethostbyname(host)
-            if host in ['0.0.0.0', '']:
+            host = aioing.normalizeHost(host)
+            if host in ('0.0.0.0',):
                 host = '127.0.0.1'
+            elif host in ("::", "0:0:0:0:0:0:0:0"):
+                host = "::1"
             eha = (host, port)
         self.eha = eha
         self.bs = bufsize
@@ -658,10 +736,10 @@ class Server(Acceptor):
         self.serviceAccepts()  # populate .axes
         while self.axes:
             cs, ca = self.axes.popleft()
-            if ca != cs.getpeername() or self.eha != cs.getsockname():
+            if ca != cs.getpeername(): #or self.eha != cs.getsockname():
                 raise ValueError("Accepted socket host addresses malformed for "
-                                 "peer ha {0} != {1}, ca {2} != {3}\n".format(
-                                     self.ha, cs.getsockname(), ca, cs.getpeername()))
+                                 "peer. eha {0} != {1}, ca {2} != {3}\n".format(
+                                     self.eha, cs.getsockname(), ca, cs.getpeername()))
             incomer = Incomer(ha=cs.getsockname(),
                               bs=self.bs,
                               ca=cs.getpeername(),
